@@ -2,7 +2,7 @@
  *
  * Copyright © 2004-2006 Bastien Nocera <hadess@hadess.net>
  * Copyright © 2002 David A. Schleef <ds@schleef.org>
- * Copyright © 2006 Christian Persch
+ * Copyright © 2006, 2009 Christian Persch
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -76,7 +76,6 @@ char * totem_pl_parser_resolve_uri (GFile *base_gfile, const char *relative_uri)
 
 typedef enum {
 	TOTEM_PLUGIN_TYPE_GMP,
-	TOTEM_PLUGIN_TYPE_COMPLEX,
 	TOTEM_PLUGIN_TYPE_NARROWSPACE,
 	TOTEM_PLUGIN_TYPE_MULLY,
 	TOTEM_PLUGIN_TYPE_CONE,
@@ -94,6 +93,7 @@ typedef GObjectClass TotemEmbeddedClass;
 
 typedef struct {
 	char *uri;
+	char *subtitle;
 	char *title;
 	int duration;
 	int starttime;
@@ -110,9 +110,12 @@ typedef struct _TotemEmbedded {
 	TotemStatusbar *statusbar;
 	TotemScrsaver *scrsaver;
 	int width, height;
+        char *user_agent;
 	const char *mimetype;
+        char *referrer_uri;
 	char *base_uri;
 	char *current_uri;
+	char *current_subtitle_uri;
 	char *href_uri;
 	char *target;
 	char *stream_uri;
@@ -188,6 +191,8 @@ void on_preferences1_activate (GtkButton *button, TotemEmbedded *emb);
 void on_copy_location1_activate (GtkButton *button, TotemEmbedded *emb);
 void on_fullscreen1_activate (GtkMenuItem *menuitem, TotemEmbedded *emb);
 
+static void update_fill (TotemEmbedded *emb, gdouble level);
+
 enum {
 	BUTTON_PRESS,
 	START_STREAM,
@@ -289,15 +294,10 @@ totem_embedded_error_and_exit (char *title, char *reason, TotemEmbedded *emb)
 }
 
 static void
-totem_embedded_save_volume (TotemEmbedded *emb, double volume)
+totem_embedded_volume_changed (TotemEmbedded *emb, double volume)
 {
-	GConfClient *gc;
 	GValue value = { 0, };
 
-	gc = gconf_client_get_default ();
-	gconf_client_set_int (gc, GCONF_PREFIX"/volume", (int) (volume * 100.0), NULL);
-	g_object_unref (G_OBJECT (gc));
-
 	g_value_init (&value, G_TYPE_DOUBLE);
 	g_value_set_double (&value, volume);
 	g_signal_emit (emb, signals[PROPERTY_CHANGE], 0,
@@ -402,7 +402,6 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 			      gboolean start_play,
 			      GError **error)
 {
-	GError *err = NULL;
 	gboolean retval;
 	const char *uri;
 
@@ -423,41 +422,17 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 		return FALSE;
 	}
 
-	g_message ("totem_embedded_open_internal '%s' is-browser-stream %d start-play %d", uri, emb->is_browser_stream, start_play);
+	g_message ("totem_embedded_open_internal '%s' subtitle '%s' is-browser-stream %d start-play %d",
+		   uri, emb->current_subtitle_uri, emb->is_browser_stream, start_play);
 
 	bacon_video_widget_set_logo_mode (emb->bvw, FALSE);
 
-	/* FIXME: remove |err| and rely on async on_error? */
-	g_message ("BEFORE _open");
-	retval = bacon_video_widget_open (emb->bvw, uri, NULL, &err);
-	g_message ("AFTER _open (ret: %d)", retval);
-	if (retval == FALSE)
-	{
-		GError *errint;
-
-		/* FIXME we haven't even started sending yet! */
-		//g_signal_emit (emb, signals[STOP_STREAM], 0);
-
-		totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
-		totem_embedded_set_error_logo (emb, NULL);
-
-		errint = g_error_new (TOTEM_EMBEDDED_ERROR_QUARK,
-				      TOTEM_EMBEDDED_OPEN_FAILED,
-				      _("Totem could not play '%s'"),
-				     emb->current_uri);
-
-		totem_embedded_set_error (emb, err->code, err->message);;
-
-		g_propagate_error (error, err);
-
-		totem_embedded_set_pp_state (emb, FALSE);
-	} else {
-		/* FIXME we shouldn't even do that here */
-		if (start_play)
-			totem_embedded_play (emb, NULL);
-		else
-			totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
-	}
+	retval = bacon_video_widget_open (emb->bvw, uri, emb->current_subtitle_uri, NULL);
+	/* FIXME we shouldn't even do that here */
+	if (start_play)
+		totem_embedded_play (emb, NULL);
+	else
+		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
 
 	totem_embedded_update_menu (emb);
 	if (emb->href_uri != NULL)
@@ -526,6 +501,8 @@ totem_embedded_do_command (TotemEmbedded *embedded,
 {
 	g_return_val_if_fail (command != NULL, FALSE);
 
+	g_message ("totem_embedded_do_command: %s", command);
+
 	if (strcmp (command, TOTEM_COMMAND_PLAY) == 0) {
 		return totem_embedded_play (embedded, error);
 	}
@@ -549,6 +526,9 @@ totem_embedded_set_href (TotemEmbedded *embedded,
 			 const char *target,
 			 GError *error)
 {
+	g_message ("totem_embedded_set_href %s (target: %s)",
+		   href_uri, target);
+
 	g_free (embedded->href_uri);
 	g_free (embedded->target);
 
@@ -577,7 +557,7 @@ totem_embedded_set_volume (TotemEmbedded *embedded,
 {
 	g_message ("totem_embedded_set_volume: %f", volume);
 	bacon_video_widget_set_volume (embedded->bvw, volume);
-	totem_embedded_save_volume (embedded, volume);
+	totem_embedded_volume_changed (embedded, volume);
 	return TRUE;
 }
 
@@ -620,13 +600,15 @@ static void
 totem_embedded_set_uri (TotemEmbedded *emb,
 		        const char *uri,
 		        const char *base_uri,
+		        const char *subtitle,
 		        gboolean is_browser_stream)
 {
 	GFile *base_gfile;
-	char *old_uri, *old_base, *old_href;
+	char *old_uri, *old_base, *old_href, *old_subtitle;
 
 	base_gfile = NULL;
 	old_uri = emb->current_uri;
+	old_subtitle = emb->current_subtitle_uri;
 	old_base = emb->base_uri;
 	old_href = emb->href_uri;
 
@@ -634,13 +616,18 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	if (base_uri)
 		base_gfile = g_file_new_for_uri (base_uri);
 	emb->current_uri = totem_pl_parser_resolve_uri (base_gfile, uri);
+	if (subtitle != NULL)
+		emb->current_subtitle_uri = totem_pl_parser_resolve_uri (base_gfile, subtitle);
+	else
+		emb->current_subtitle_uri = NULL;
 	if (base_gfile)
 		g_object_unref (base_gfile);
 	emb->is_browser_stream = (is_browser_stream != FALSE);
 	emb->href_uri = NULL;
 
 	if (uri != NULL)
-		g_print ("totem_embedded_set_uri uri %s base %s => resolved %s\n", uri, base_uri, emb->current_uri);
+		g_print ("totem_embedded_set_uri uri %s base %s => resolved %s (subtitle %s => resolved %s)\n",
+			 uri, base_uri, emb->current_uri, subtitle, emb->current_subtitle_uri);
 	else
 		g_print ("Emptying current_uri\n");
 
@@ -653,6 +640,7 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	g_free (old_uri);
 	g_free (old_base);
 	g_free (old_href);
+	g_free (old_subtitle);
 	g_free (emb->stream_uri);
 	emb->stream_uri = NULL;
 }
@@ -676,12 +664,15 @@ totem_pl_item_free (gpointer data, gpointer user_data)
 		return;
 	g_free (item->uri);
 	g_free (item->title);
-	g_free (item);
+	g_free (item->subtitle);
+	g_slice_free (TotemPlItem, item);
 }
 
 static gboolean
 totem_embedded_clear_playlist (TotemEmbedded *emb, GError *error)
 {
+	g_message ("totem_embedded_clear_playlist");
+
 	g_list_foreach (emb->playlist, (GFunc) totem_pl_item_free, NULL);
 	g_list_free (emb->playlist);
 
@@ -689,23 +680,32 @@ totem_embedded_clear_playlist (TotemEmbedded *emb, GError *error)
 	emb->current = NULL;
 	emb->num_items = 0;
 
-	totem_embedded_set_uri (emb, NULL, NULL, FALSE);
+	totem_embedded_set_uri (emb, NULL, NULL, NULL, FALSE);
 
 	bacon_video_widget_close (emb->bvw);
+	update_fill (emb, -1.0);
 	totem_embedded_update_title (emb, NULL);
 
 	return TRUE;
 }
 
 static gboolean
-totem_embedded_add_item (TotemEmbedded *embedded, const char *uri, GError *error)
+totem_embedded_add_item (TotemEmbedded *embedded,
+			 const char *base_uri,
+			 const char *uri,
+			 const char *title,
+			 const char *subtitle,
+			 GError *error)
 {
 	TotemPlItem *item;
 
-	g_message ("totem_embedded_add_item: %s", uri);
+	g_message ("totem_embedded_add_item: %s (base: %s title: %s subtitle: %s)",
+		   uri, base_uri, title, subtitle);
 
-	item = g_new0 (TotemPlItem, 1);
+	item = g_slice_new0 (TotemPlItem);
 	item->uri = g_strdup (uri);
+	item->title = g_strdup (title);
+	item->subtitle = g_strdup (subtitle);
 	item->duration = -1;
 	item->starttime = -1;
 
@@ -716,7 +716,8 @@ totem_embedded_add_item (TotemEmbedded *embedded, const char *uri, GError *error
 		embedded->current = embedded->playlist;
 		totem_embedded_set_uri (embedded,
 					(const char *) uri,
-					embedded->base_uri /* FIXME? */,
+					base_uri,
+					subtitle,
 					FALSE);
 		totem_embedded_open_internal (embedded, FALSE, NULL /* FIXME */);
 	}
@@ -790,7 +791,7 @@ totem_embedded_open_uri (TotemEmbedded *emb,
 
 	totem_embedded_clear_playlist (emb, NULL);
 
-	totem_embedded_set_uri (emb, uri, base_uri, FALSE);
+	totem_embedded_set_uri (emb, uri, base_uri, NULL, FALSE);
 	/* We can only have one item in the "playlist" when
 	 * we open a particular URI like this */
 	emb->num_items = 1;
@@ -808,7 +809,7 @@ totem_embedded_open_stream (TotemEmbedded *emb,
 
 	totem_embedded_clear_playlist (emb, NULL);
 
-	totem_embedded_set_uri (emb, uri, base_uri, TRUE);
+	totem_embedded_set_uri (emb, uri, base_uri, NULL, TRUE);
 	/* We can only have one item in the "playlist" when
 	 * we open a browser stream */
 	emb->num_items = 1;
@@ -822,11 +823,14 @@ static gboolean
 totem_embedded_close_stream (TotemEmbedded *emb,
 			     GError *error)
 {
+	g_message ("totem_embedded_close_stream");
+
 	if (!emb->is_browser_stream)
 		return TRUE;
 
 	/* FIXME this enough? */
 	bacon_video_widget_close (emb->bvw);
+	update_fill (emb, -1.0);
 
 	return TRUE;
 }
@@ -856,9 +860,13 @@ totem_embedded_open_playlist_item (TotemEmbedded *emb,
 	totem_embedded_set_uri (emb,
 				(const char *) plitem->uri,
 			        emb->base_uri /* FIXME? */,
+			        plitem->subtitle,
 			        FALSE);
 
 	bacon_video_widget_close (emb->bvw);
+	update_fill (emb, -1.0);
+
+	//FIXME set the title from the URI if possible
 	totem_embedded_update_title (emb, plitem->title);
 	if (totem_embedded_open_internal (emb, FALSE, NULL /* FIXME */)) {
 		if (plitem->starttime > 0) {
@@ -889,7 +897,8 @@ totem_embedded_set_local_file (TotemEmbedded *emb,
 {
 	char *file_uri;
 
-	g_message ("Setting the current path to %s", path);
+	g_message ("Setting the current path to %s (uri: %s base: %s)",
+		   path, uri, base_uri);
 
 	totem_embedded_clear_playlist (emb, NULL);
 
@@ -898,7 +907,7 @@ totem_embedded_set_local_file (TotemEmbedded *emb,
 		return FALSE;
 
 	/* FIXME what about |uri| param?!! */
-	totem_embedded_set_uri (emb, file_uri, base_uri, FALSE);
+	totem_embedded_set_uri (emb, file_uri, base_uri, emb->current_subtitle_uri, FALSE);
 	g_free (file_uri);
 
 	return totem_embedded_open_internal (emb, TRUE, error);
@@ -911,6 +920,8 @@ totem_embedded_set_local_cache (TotemEmbedded *emb,
 {
 	int fd;
 
+	g_message ("totem_embedded_set_local_cache: %s", path);
+
 	/* FIXME Should also handle playlists */
 	if (!emb->is_browser_stream)
 		return TRUE;
@@ -943,8 +954,8 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 	GFile *src, *dst;
 	int fd;
 
-	g_message ("Setting the current playlist to %s (base: %s)",
-		   path, base_uri);
+	g_message ("Setting the current playlist to %s (uri: %s base: %s)",
+		   path, uri, base_uri);
 
 	totem_embedded_clear_playlist (emb, NULL);
 
@@ -962,7 +973,7 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 	}
 	src = g_file_new_for_path (path);
 	dst = g_file_new_for_path (tmpfile);
-	if (g_file_copy (src, dst, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &err) == FALSE) {
+	if (g_file_copy (src, dst, G_FILE_COPY_OVERWRITE | G_FILE_COPY_TARGET_DEFAULT_PERMS, NULL, NULL, NULL, &err) == FALSE) {
 		g_warning ("Failed to copy playlist '%s' to '%s': %s",
 			   path, tmpfile, err->message);
 		g_error_free (err);
@@ -980,7 +991,7 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 	g_object_unref (dst);
 	close (fd);
 
-	totem_embedded_set_uri (emb, file_uri, base_uri, FALSE);
+	totem_embedded_set_uri (emb, file_uri, base_uri, NULL, FALSE);
 	g_free (file_uri);
 
 	/* Schedule parsing on idle */
@@ -1260,6 +1271,16 @@ on_popup_button_button_pressed (GtkToggleButton *button,
 	return FALSE;
 }
 
+static void
+on_popup_menu_unmap (GtkWidget *menu,
+		     TotemEmbedded *emb)
+{
+	GtkWidget *button;
+
+	button = GTK_WIDGET (gtk_builder_get_object (emb->xml, "popup_button"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+}
+
 static char *
 resolve_redirect (const char *old_mrl, const char *mrl)
 {
@@ -1284,7 +1305,7 @@ resolve_redirect (const char *old_mrl, const char *mrl)
 static void
 on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 {
-	char *new_uri;
+	char *new_uri = NULL;
 
 	g_message ("stream uri: %s", emb->stream_uri ? emb->stream_uri : "(null)");
 	g_message ("current uri: %s", emb->current_uri ? emb->current_uri : "(null)");
@@ -1292,12 +1313,13 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 	g_message ("redirect: %s", mrl);
 
 	bacon_video_widget_close (emb->bvw);
+	update_fill (emb, -1.0);
 
 	/* If we don't have a relative URI */
 	if (strstr (mrl, "://") != NULL)
 		new_uri = NULL;
 	/* We are using a local cache, so we resolve against the stream_uri */
-	if (emb->stream_uri)
+	else if (emb->stream_uri)
 		new_uri = resolve_redirect (emb->stream_uri, mrl);
 	/* We don't have a local cache, so resolve against the URI */
 	else if (emb->current_uri)
@@ -1311,7 +1333,7 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 	/* FIXME: clear playlist? or replace current entry? or add a new entry? */
 	/* FIXME: use totem_embedded_open_uri? */
 
-	totem_embedded_set_uri (emb, new_uri ? new_uri : mrl , emb->base_uri /* FIXME? */, FALSE);
+	totem_embedded_set_uri (emb, new_uri ? new_uri : mrl , emb->base_uri /* FIXME? */, emb->current_subtitle_uri, FALSE);
 
 	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 
@@ -1446,6 +1468,7 @@ on_eos_event (BaconVideoWidget *bvw, TotemEmbedded *emb)
 			emb->num_items = 1;
 			emb->is_browser_stream = FALSE;
 			bacon_video_widget_close (emb->bvw);
+			update_fill (emb, -1.0);
 			totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 		} else {
 			/* FIXME: should find a way to enable playback of the stream again without re-requesting it */
@@ -1461,10 +1484,12 @@ on_eos_event (BaconVideoWidget *bvw, TotemEmbedded *emb)
 					totem_embedded_play (emb, NULL);
 			} else {
 				bacon_video_widget_close (emb->bvw);
+				update_fill (emb, -1.0);
 				totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 			}
 		} else {
 			bacon_video_widget_close (emb->bvw);
+			update_fill (emb, -1.0);
 			totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 		}
 	} else if (emb->current) {
@@ -1525,7 +1550,7 @@ static void
 cb_vol (GtkWidget *val, gdouble value, TotemEmbedded *emb)
 {
 	bacon_video_widget_set_volume (emb->bvw, value);
-	totem_embedded_save_volume (emb, value);
+	totem_embedded_volume_changed (emb, value);
 }
 
 static void
@@ -1568,6 +1593,27 @@ on_buffering (BaconVideoWidget *bvw, guint percentage, TotemEmbedded *emb)
 //	g_message ("Buffering: %d %%", percentage);
 }
 
+static void
+update_fill (TotemEmbedded *emb, gdouble level)
+{
+	if (level < 0.0) {
+		gtk_range_set_show_fill_level (GTK_RANGE (emb->seek), FALSE);
+		gtk_range_set_show_fill_level (GTK_RANGE (emb->fs->seek), FALSE);
+	} else {
+		gtk_range_set_fill_level (GTK_RANGE (emb->seek), level * 65535.0f);
+		gtk_range_set_show_fill_level (GTK_RANGE (emb->seek), TRUE);
+
+		gtk_range_set_fill_level (GTK_RANGE (emb->fs->seek), level * 65535.0f);
+		gtk_range_set_show_fill_level (GTK_RANGE (emb->fs->seek), TRUE);
+	}
+}
+
+static void
+on_download_buffering (BaconVideoWidget *bvw, gdouble level, TotemEmbedded *emb)
+{
+	update_fill (emb, level);
+}
+
 static void
 property_notify_cb_volume (BaconVideoWidget *bvw,
 			   GParamSpec *spec,
@@ -1579,7 +1625,7 @@ property_notify_cb_volume (BaconVideoWidget *bvw,
 	
 	g_signal_handlers_block_by_func (emb->volume, cb_vol, emb);
 	gtk_scale_button_set_value (GTK_SCALE_BUTTON (emb->volume), volume);
-	totem_embedded_save_volume (emb, volume);
+	totem_embedded_volume_changed (emb, volume);
 	g_signal_handlers_unblock_by_func (emb->volume, cb_vol, emb);
 }
 
@@ -1635,7 +1681,7 @@ totem_embedded_action_volume_relative (TotemEmbedded *emb, double off_pct)
 	
 	vol = bacon_video_widget_get_volume (emb->bvw);
 	bacon_video_widget_set_volume (emb->bvw, vol + off_pct);
-	totem_embedded_save_volume (emb, vol + off_pct);
+	totem_embedded_volume_changed (emb, vol + off_pct);
 }
 
 static gboolean
@@ -1703,6 +1749,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 {
 	GtkWidget *child, *container, *image;
 	GtkWidget *popup_button;
+	GtkWidget *menu;
 	BvwUseType type;
 	GError *err = NULL;
 	GConfClient *gc;
@@ -1742,16 +1789,24 @@ totem_embedded_construct (TotemEmbedded *emb,
 					       (-1, -1, type, &err));
 	}
 
-	/* FIXME! */
-	if (emb->bvw == NULL) {
-		/* FIXME! */
-		/* FIXME construct and show error message */
-		totem_embedded_error_and_exit (_("The Totem plugin could not be started."), err != NULL ? err->message : _("No reason."), emb);
-
-		if (err != NULL)
-			g_error_free (err);
+	/* FIXME: check the UA strings of the legacy plugins themselves */
+	/* FIXME: at least hxplayer seems to send different UAs depending on the protocol!? */
+	if (emb->user_agent != NULL) {
+                bacon_video_widget_set_user_agent (emb->bvw, emb->user_agent);
+		g_free (emb->user_agent);
+		emb->user_agent = NULL;
 	}
 
+        /* FIXMEchpe: this is just the initial value. I think in case e.g. we're doing
+         * a playlist, the playlist's URI should become the referrer?
+         */
+        g_print ("Referrer URI: %s\n", emb->referrer_uri);
+        if (emb->referrer_uri != NULL) {
+                bacon_video_widget_set_referrer (emb->bvw, emb->referrer_uri);
+                g_free (emb->referrer_uri);
+                emb->referrer_uri = NULL;
+        }
+
 	/* Fullscreen setup */
 	emb->fs_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_widget_realize (emb->fs_window);
@@ -1797,6 +1852,8 @@ totem_embedded_construct (TotemEmbedded *emb,
 			G_CALLBACK (on_tick), emb);
 	g_signal_connect (G_OBJECT (emb->bvw), "buffering",
 			  G_CALLBACK (on_buffering), emb);
+	g_signal_connect (G_OBJECT (emb->bvw), "download-buffering",
+			  G_CALLBACK (on_download_buffering), emb);
 
 	g_signal_connect (G_OBJECT (emb->bvw), "notify::volume",
 			  G_CALLBACK (property_notify_cb_volume), emb);
@@ -1838,6 +1895,9 @@ totem_embedded_construct (TotemEmbedded *emb,
 			  G_CALLBACK (on_popup_button_toggled), emb);
 	g_signal_connect (G_OBJECT (popup_button), "button-press-event",
 			  G_CALLBACK (on_popup_button_button_pressed), emb);
+	menu = GTK_WIDGET (gtk_builder_get_object (emb->menuxml, "menu"));
+	g_signal_connect (G_OBJECT (menu), "unmap",
+			  G_CALLBACK (on_popup_menu_unmap), emb);
 
 	gc = gconf_client_get_default ();
 	volume = ((double) gconf_client_get_int (gc, GCONF_PREFIX"/volume", NULL)) / 100.0;
@@ -1852,7 +1912,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 	g_signal_connect (G_OBJECT (emb->volume), "value-changed",
 			  G_CALLBACK (cb_vol), emb);
 	bacon_video_widget_set_volume (emb->bvw, volume);
-	totem_embedded_save_volume (emb, volume);
+	totem_embedded_volume_changed (emb, volume);
 
 	emb->statusbar = TOTEM_STATUSBAR (gtk_builder_get_object (emb->xml, "statusbar"));
 	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (emb->statusbar), FALSE);
@@ -1924,29 +1984,6 @@ totem_embedded_construct (TotemEmbedded *emb,
 	return TRUE;
 }
 
-static gboolean
-totem_embedded_construct_placeholder (TotemEmbedded *emb,
-			  GdkNativeWindow xid,
-			  int width,
-			  int height)
-{
-	GtkWidget *window, *label;
-
-	if (xid == 0)
-		return TRUE;
-
-	window = gtk_plug_new (xid);
-	/* FIXME why? */
-	gtk_widget_realize (window);
-
-	label = gtk_label_new ("Not yet supported");
-	gtk_container_add (GTK_CONTAINER (window), label);
-
-	gtk_widget_show_all (window);
-
-	return TRUE;
-}
-
 static gboolean
 totem_embedded_set_window (TotemEmbedded *embedded,
 			   const char *controls,
@@ -1957,22 +1994,13 @@ totem_embedded_set_window (TotemEmbedded *embedded,
 {
 	g_print ("Viewer: SetWindow XID %u size %d:%d\n", window, width, height);
 
-	if (embedded->type == TOTEM_PLUGIN_TYPE_COMPLEX) {
-		/* FIXME!!! */
-		if (strcmp (controls, "All") != 0 &&
-		    strcmp (controls, "ImageWindow") != 0) {
-			totem_embedded_construct_placeholder (embedded, (GdkNativeWindow) window, width, height);
-			return TRUE;
-		}
-	} else {
-		if (strcmp (controls, "All") != 0 &&
-		    strcmp (controls, "ImageWindow") != 0) {
-			g_set_error (error,
-				TOTEM_EMBEDDED_ERROR_QUARK,
-				TOTEM_EMBEDDED_SETWINDOW_UNSUPPORTED_CONTROLS,
-				"Unsupported controls '%s'", controls);
-			return FALSE;
-		}
+	if (strcmp (controls, "All") != 0 &&
+	    strcmp (controls, "ImageWindow") != 0) {
+		g_set_error (error,
+			     TOTEM_EMBEDDED_ERROR_QUARK,
+			     TOTEM_EMBEDDED_SETWINDOW_UNSUPPORTED_CONTROLS,
+			     "Unsupported controls '%s'", controls);
+		return FALSE;
 	}
 
 	if (embedded->window != NULL) {
@@ -2045,7 +2073,7 @@ entry_parsed (TotemPlParser *parser,
 	if (duration == 0)
 		return;
 
-	item = g_new0 (TotemPlItem, 1);
+	item = g_slice_new0 (TotemPlItem);
 	item->uri = g_strdup (uri);
 	item->title = g_strdup (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE));
 	item->duration = duration;
@@ -2123,6 +2151,7 @@ totem_embedded_push_parser (gpointer data)
 static char *arg_user_agent = NULL;
 static char *arg_mime_type = NULL;
 static char **arg_remaining = NULL;
+static char *arg_referrer = NULL;
 static gboolean arg_no_controls = FALSE;
 static gboolean arg_statusbar = FALSE;
 static gboolean arg_hidden = FALSE;
@@ -2140,7 +2169,6 @@ parse_plugin_type (const gchar *option_name,
 {
 	const char types[TOTEM_PLUGIN_TYPE_LAST][12] = {
 		"gmp",
-		"complex",
 		"narrowspace",
 		"mully",
 		"cone"
@@ -2170,6 +2198,7 @@ static GOptionEntry option_entries [] =
 	{ TOTEM_OPTION_REPEAT, 0, 0, G_OPTION_ARG_NONE, &arg_repeat, NULL, NULL },
 	{ TOTEM_OPTION_NOAUTOSTART, 0, 0, G_OPTION_ARG_NONE, &arg_no_autostart, NULL, NULL },
 	{ TOTEM_OPTION_AUDIOONLY, 0, 0, G_OPTION_ARG_NONE, &arg_audioonly, NULL, NULL },
+        { TOTEM_OPTION_REFERRER, 0, 0, G_OPTION_ARG_STRING, &arg_referrer, NULL, NULL },
 	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY /* STRING? */, &arg_remaining, NULL },
 	{ NULL }
 };
@@ -2267,14 +2296,6 @@ int main (int argc, char **argv)
 		exit (1);
 	}
 
-	/* FIXME: check the UA strings of the legacy plugins themselves */
-	/* FIXME: at least hxplayer seems to send different UAs depending on the protocol!? */
-	if (arg_user_agent != NULL) {
-		g_setenv ("BACON_VIDEO_WIDGET_HTTP_USER_AGENT", arg_user_agent, TRUE);
-		g_free (arg_user_agent);
-		arg_user_agent = NULL;
-	}
-
 	bacon_video_widget_init_backend (NULL, NULL);
 
 	dbus_g_object_type_install_info (TOTEM_TYPE_EMBEDDED,
@@ -2322,6 +2343,8 @@ int main (int argc, char **argv)
 	emb->autostart = !arg_no_autostart;
 	emb->audioonly = arg_audioonly;
 	emb->type = arg_plugin_type;
+        emb->user_agent = arg_user_agent;
+        emb->referrer_uri = arg_referrer;
 
 	/* FIXME: register this BEFORE requesting the service name? */
 	dbus_g_connection_register_g_object
