@@ -47,9 +47,11 @@ static void bacon_resize_finalize	(GObject *object);
 
 static void set_video_widget		(BaconResize *resize,
 					 GtkWidget *video_widget);
+#ifdef HAVE_XVIDMODE
 static void screen_changed_cb		(GtkWidget *video_widget,
 					 GdkScreen *previous_screen,
 					 BaconResize *resize);
+#endif /* HAVE_XVIDMODE */
 
 struct BaconResizePrivate {
 	gboolean have_xvidmode;
@@ -86,13 +88,15 @@ bacon_resize_class_init (BaconResizeClass *klass)
 
 	g_object_class_install_property (object_class, PROP_HAVE_XVIDMODE,
 					 g_param_spec_boolean ("have-xvidmode", NULL, NULL,
-							       FALSE, G_PARAM_READABLE));
+							       FALSE, G_PARAM_READABLE |
+                                                               G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (object_class, PROP_VIDEO_WIDGET,
 					 g_param_spec_object ("video-widget", "video-widget",
 						 	      "The related video widget",
 							      GTK_TYPE_WIDGET,
-							      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY |
+                                                              G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -198,15 +202,15 @@ bail:
 #endif /* HAVE_XVIDMODE */
 }
 
+#ifdef HAVE_XVIDMODE
 static void screen_changed_cb (GtkWidget *video_widget, GdkScreen *previous_screen, BaconResize *resize)
 {
-#ifdef HAVE_XVIDMODE
 	if (resize->priv->resized == TRUE)
 		bacon_resize_resize (resize);
 	else
 		bacon_resize_restore (resize);
-#endif /* HAVE_XVIDMODE */
 }
+#endif /* HAVE_XVIDMODE */
 
 void
 bacon_resize_resize (BaconResize *resize)
@@ -218,16 +222,16 @@ bacon_resize_resize (BaconResize *resize)
 	gboolean found = FALSE;
 	GdkWindow *root;
 	GdkScreen *screen;
-	Display *Display;
+	Display *xdisplay;
 
 	g_return_if_fail (GTK_IS_WIDGET (resize->priv->video_widget));
 	g_return_if_fail (GTK_WIDGET_REALIZED (resize->priv->video_widget));
 
-	Display = GDK_DRAWABLE_XDISPLAY (resize->priv->video_widget->window);
-	if (Display == NULL)
+	xdisplay = GDK_DRAWABLE_XDISPLAY (resize->priv->video_widget->window);
+	if (xdisplay == NULL)
 		return;
 
-	XLockDisplay (Display);
+	XLockDisplay (xdisplay);
 
 	screen = gtk_widget_get_screen (resize->priv->video_widget);
 	root = gdk_screen_get_root_window (screen);
@@ -236,7 +240,7 @@ bacon_resize_resize (BaconResize *resize)
 	if (gdk_screen_get_n_monitors (screen) > 1)
 		goto bail;
 
-	res = XF86VidModeGetModeLine (Display, GDK_SCREEN_XNUMBER (screen), &dotclock, &modeline);
+	res = XF86VidModeGetModeLine (xdisplay, GDK_SCREEN_XNUMBER (screen), &dotclock, &modeline);
 	if (!res)
 		goto bail;
 
@@ -250,7 +254,7 @@ bacon_resize_resize (BaconResize *resize)
 	gdk_error_trap_push ();
 
 	/* Find the XRandR mode that corresponds to the real size */
-	resize->priv->xr_screen_conf = XRRGetScreenInfo (Display, GDK_WINDOW_XWINDOW (root));
+	resize->priv->xr_screen_conf = XRRGetScreenInfo (xdisplay, GDK_WINDOW_XWINDOW (root));
 	xr_sizes = XRRConfigSizes (resize->priv->xr_screen_conf, &xr_nsize);
 	resize->priv->xr_original_size = XRRConfigCurrentConfiguration (resize->priv->xr_screen_conf, &(resize->priv->xr_current_rotation));
 	if (gdk_error_trap_pop ()) {
@@ -269,7 +273,7 @@ bacon_resize_resize (BaconResize *resize)
 		goto bail;
 
 	gdk_error_trap_push ();
-	XRRSetScreenConfig (Display,
+	XRRSetScreenConfig (xdisplay,
 			resize->priv->xr_screen_conf,
 			GDK_WINDOW_XWINDOW (root),
 			(SizeID) i,
@@ -282,7 +286,7 @@ bacon_resize_resize (BaconResize *resize)
 		resize->priv->resized = TRUE;
 
 bail:
-	XUnlockDisplay (Display);
+	XUnlockDisplay (xdisplay);
 #endif /* HAVE_XVIDMODE */
 }
 
@@ -294,7 +298,7 @@ bacon_resize_restore (BaconResize *resize)
 	XF86VidModeModeLine modeline;
 	GdkWindow *root;
 	GdkScreen *screen;
-	Display *Display;
+	Display *xdisplay;
 
 	g_return_if_fail (GTK_IS_WIDGET (resize->priv->video_widget));
 	g_return_if_fail (GTK_WIDGET_REALIZED (resize->priv->video_widget));
@@ -304,15 +308,15 @@ bacon_resize_restore (BaconResize *resize)
 	if (resize->priv->xr_screen_conf == NULL)
 		return;
 
-	Display = GDK_DRAWABLE_XDISPLAY (resize->priv->video_widget->window);
-	if (Display == NULL)
+	xdisplay = GDK_DRAWABLE_XDISPLAY (resize->priv->video_widget->window);
+	if (xdisplay == NULL)
 		return;
 
-	XLockDisplay (Display);
+	XLockDisplay (xdisplay);
 
 	screen = gtk_widget_get_screen (resize->priv->video_widget);
 	root = gdk_screen_get_root_window (screen);
-	res = XF86VidModeGetModeLine (Display, GDK_SCREEN_XNUMBER (screen), &dotclock, &modeline);
+	res = XF86VidModeGetModeLine (xdisplay, GDK_SCREEN_XNUMBER (screen), &dotclock, &modeline);
 	if (!res)
 		goto bail;
 
@@ -324,7 +328,7 @@ bacon_resize_restore (BaconResize *resize)
 		goto bail;
 
 	gdk_error_trap_push ();
-	XRRSetScreenConfig (Display,
+	XRRSetScreenConfig (xdisplay,
 			resize->priv->xr_screen_conf,
 			GDK_WINDOW_XWINDOW (root),
 			resize->priv->xr_original_size,
@@ -340,7 +344,7 @@ bacon_resize_restore (BaconResize *resize)
 	resize->priv->xr_screen_conf = NULL;
 
 bail:
-	XUnlockDisplay (Display);
+	XUnlockDisplay (xdisplay);
 #endif
 }
 
