@@ -74,6 +74,7 @@ typedef struct {
 	void (*tick) (GtkWidget *bvw, gint64 current_time, gint64 stream_length,
 			double current_position, gboolean seekable);
 	void (*buffering) (GtkWidget *bvw, guint progress);
+	void (*download_buffering) (GtkWidget *bvw, gdouble percentage);
 } BaconVideoWidgetClass;
 
 /**
@@ -183,6 +184,9 @@ gboolean bacon_video_widget_seek		 (BaconVideoWidget *bvw,
 gboolean bacon_video_widget_seek_time		 (BaconVideoWidget *bvw,
 						  gint64 time,
 						  GError **error);
+gboolean bacon_video_widget_step		 (BaconVideoWidget *bvw,
+						  gboolean forward,
+						  GError **error);
 gboolean bacon_video_widget_can_direct_seek	 (BaconVideoWidget *bvw);
 double bacon_video_widget_get_position           (BaconVideoWidget *bvw);
 gint64 bacon_video_widget_get_current_time       (BaconVideoWidget *bvw);
@@ -199,9 +203,7 @@ double bacon_video_widget_get_volume             (BaconVideoWidget *bvw);
 
 /* Properties */
 void bacon_video_widget_set_logo		 (BaconVideoWidget *bvw,
-						  char *filename);
-void bacon_video_widget_set_logo_pixbuf		 (BaconVideoWidget *bvw,
-						  GdkPixbuf *logo);
+						  const char *name);
 void  bacon_video_widget_set_logo_mode		 (BaconVideoWidget *bvw,
 						  gboolean logo_mode);
 gboolean bacon_video_widget_get_logo_mode	 (BaconVideoWidget *bvw);
@@ -230,6 +232,12 @@ void bacon_video_widget_set_subtitle_font	 (BaconVideoWidget *bvw,
 void bacon_video_widget_set_subtitle_encoding	 (BaconVideoWidget *bvw,
 						  const char *encoding);
 
+void bacon_video_widget_set_user_agent           (BaconVideoWidget *bvw,
+                                                  const char *user_agent);
+
+void bacon_video_widget_set_referrer             (BaconVideoWidget *bvw,
+                                                  const char *referrer);
+
 /* Metadata */
 /**
  * BvwMetadataType:
@@ -372,6 +380,8 @@ void bacon_video_widget_set_video_property       (BaconVideoWidget *bvw,
 						  BvwVideoProperty type,
 						  int value);
 
+gboolean bacon_video_widget_has_menus            (BaconVideoWidget *bvw);
+
 /* DVD functions */
 /**
  * BvwDVDEvent:
