@@ -37,12 +37,12 @@
 #define SPACING 4
 #define NORMAL_CONTEXT "text"
 #define BUFFERING_CONTEXT "buffering"
+#define HELP_CONTEXT "help"
 
-static void totem_statusbar_class_init       (TotemStatusbarClass *class);
-static void totem_statusbar_init             (TotemStatusbar      *statusbar);
 static void totem_statusbar_dispose          (GObject             *object);
 static void totem_statusbar_sync_description (TotemStatusbar      *statusbar);
 
+G_MODULE_EXPORT GType totem_statusbar_get_type (void);
 G_DEFINE_TYPE(TotemStatusbar, totem_statusbar, GTK_TYPE_STATUSBAR)
 
 static void
@@ -62,16 +62,10 @@ totem_statusbar_init (TotemStatusbar *statusbar)
   statusbar->time = 0;
   statusbar->length = -1;
 
-  hbox = gtk_hbox_new (FALSE, SPACING);
-
-  /* Widget surgery */
-  g_object_ref (gstatusbar->label);
-  gtk_container_remove (GTK_CONTAINER (gstatusbar->frame), gstatusbar->label);
-  gtk_box_pack_start (GTK_BOX (hbox), gstatusbar->label, FALSE, FALSE, 0);
-  g_object_unref (gstatusbar->label);
-  gtk_container_add (GTK_CONTAINER (gstatusbar->frame), hbox);
-  gtk_widget_show (hbox);
+  hbox = gtk_statusbar_get_message_area (gstatusbar);
 
+  gtk_box_set_child_packing (GTK_BOX (hbox), gstatusbar->label,
+			     FALSE, FALSE, 0, GTK_PACK_START);
   gtk_label_set_ellipsize (GTK_LABEL (gstatusbar->label), FALSE);
 
   /* progressbar for network streams */
@@ -109,26 +103,26 @@ totem_statusbar_new (void)
 static void
 totem_statusbar_update_time (TotemStatusbar *statusbar)
 {
-  char *time, *length, *label;
+  char *time_string, *length, *label;
 
-  time = totem_time_to_string (statusbar->time * 1000);
+  time_string = totem_time_to_string (statusbar->time * 1000);
 
   if (statusbar->length < 0) {
-    label = g_strdup_printf (_("%s (Streaming)"), time);
+    label = g_strdup_printf (_("%s (Streaming)"), time_string);
   } else {
     length = totem_time_to_string
 	    (statusbar->length == -1 ? 0 : statusbar->length * 1000);
 
     if (statusbar->seeking == FALSE)
       /* Elapsed / Total Length */
-      label = g_strdup_printf (_("%s / %s"), time, length);
+      label = g_strdup_printf (_("%s / %s"), time_string, length);
     else
       /* Seeking to Time / Total Length */
-      label = g_strdup_printf (_("Seek to %s / %s"), time, length);
+      label = g_strdup_printf (_("Seek to %s / %s"), time_string, length);
 
     g_free (length);
   }
-  g_free (time);
+  g_free (time_string);
 
   gtk_label_set_text (GTK_LABEL (statusbar->time_label), label);
   g_free (label);
@@ -150,17 +144,39 @@ totem_statusbar_set_text (TotemStatusbar *statusbar, const char *label)
 }
 
 void
-totem_statusbar_set_time (TotemStatusbar *statusbar, gint time)
+totem_statusbar_set_time (TotemStatusbar *statusbar, gint _time)
 {
   g_return_if_fail (TOTEM_IS_STATUSBAR (statusbar));
 
-  if (statusbar->time == time)
+  if (statusbar->time == _time)
     return;
 
-  statusbar->time = time;
+  statusbar->time = _time;
   totem_statusbar_update_time (statusbar);
 }
 
+/* Set a help message to be displayed in the status bar. */
+void
+totem_statusbar_push_help (TotemStatusbar *statusbar, const char *message)
+{
+  GtkStatusbar *gstatusbar = GTK_STATUSBAR (statusbar);
+  guint id;
+
+  id = gtk_statusbar_get_context_id (gstatusbar, HELP_CONTEXT);
+  gtk_statusbar_push (gstatusbar, id, message);
+}
+
+/* Remove the last help message of the status bar. */
+void
+totem_statusbar_pop_help (TotemStatusbar *statusbar)
+{
+  GtkStatusbar *gstatusbar = GTK_STATUSBAR (statusbar);
+  guint id;
+
+  id = gtk_statusbar_get_context_id (gstatusbar, HELP_CONTEXT);
+  gtk_statusbar_pop (gstatusbar, id);
+}
+
 static gboolean
 totem_statusbar_timeout_pop (TotemStatusbar *statusbar)
 {
@@ -240,13 +256,13 @@ totem_statusbar_pop (TotemStatusbar *statusbar)
 
 void
 totem_statusbar_set_time_and_length (TotemStatusbar *statusbar,
-				     gint time, gint length)
+				     gint _time, gint length)
 {
   g_return_if_fail (TOTEM_IS_STATUSBAR (statusbar));
 
-  if (time != statusbar->time ||
+  if (_time != statusbar->time ||
       length != statusbar->length) {
-    statusbar->time = time;
+    statusbar->time = _time;
     statusbar->length = length;
 
     totem_statusbar_update_time (statusbar);
